@@ -1,8 +1,9 @@
 from .Exporter import Exporter
-from pyqtgraph.python2_3 import asUnicode
-from pyqtgraph.parametertree import Parameter
-from pyqtgraph.Qt import QtGui, QtCore, QtSvg
-import pyqtgraph as pg
+from ..python2_3 import asUnicode
+from ..parametertree import Parameter
+from ..Qt import QtGui, QtCore, QtSvg, USE_PYSIDE
+from .. import debug
+from .. import functions as fn
 import re
 import xml.dom.minidom as xml
 import numpy as np
@@ -101,14 +102,12 @@ xmlHeader = """\
 <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"  version="1.2" baseProfile="tiny">
 <title>pyqtgraph SVG export</title>
 <desc>Generated with Qt and pyqtgraph</desc>
-<defs>
-</defs>
 """
 
 def generateSvg(item):
     global xmlHeader
     try:
-        node = _generateItemSvg(item)
+        node, defs = _generateItemSvg(item)
     finally:
         ## reset export mode for all items in the tree
         if isinstance(item, QtGui.QGraphicsScene):
@@ -123,7 +122,11 @@ def generateSvg(item):
     
     cleanXml(node)
     
-    return xmlHeader + node.toprettyxml(indent='    ') + "\n</svg>\n"
+    defsXml = "<defs>\n"
+    for d in defs:
+        defsXml += d.toprettyxml(indent='    ')
+    defsXml += "</defs>\n"
+    return xmlHeader + defsXml + node.toprettyxml(indent='    ') + "\n</svg>\n"
 
 
 def _generateItemSvg(item, nodes=None, root=None):
@@ -156,7 +159,7 @@ def _generateItemSvg(item, nodes=None, root=None):
     ##    
     ##    Both 2 and 3 can be addressed by drawing all items in world coordinates.
     
-    prof = pg.debug.Profiler('generateItemSvg %s' % str(item), disabled=True)
+    profiler = debug.Profiler()
     
     if nodes is None:  ## nodes maps all node IDs to their XML element. 
                        ## this allows us to ensure all elements receive unique names.
@@ -196,17 +199,12 @@ def _generateItemSvg(item, nodes=None, root=None):
         tr2 = QtGui.QTransform()
         tr2.translate(-rootPos.x(), -rootPos.y())
         tr = tr * tr2
-        #print item, pg.SRTTransform(tr)
 
-        #tr.translate(item.pos().x(), item.pos().y())
-        #tr = tr * item.transform()
         arr = QtCore.QByteArray()
         buf = QtCore.QBuffer(arr)
         svg = QtSvg.QSvgGenerator()
         svg.setOutputDevice(buf)
         dpi = QtGui.QDesktopWidget().physicalDpiX()
-        ### not really sure why this works, but it seems to be important:
-        #self.svg.setSize(QtCore.QSize(self.params['width']*dpi/90., self.params['height']*dpi/90.))
         svg.setResolution(dpi)
 
         p = QtGui.QPainter()
@@ -223,7 +221,10 @@ def _generateItemSvg(item, nodes=None, root=None):
             #if hasattr(item, 'setExportMode'):
                 #item.setExportMode(False)
 
-        xmlStr = bytes(arr).decode('utf-8')
+        if USE_PYSIDE:
+            xmlStr = str(arr)
+        else:
+            xmlStr = bytes(arr).decode('utf-8')
         doc = xml.parseString(xmlStr)
         
     try:
@@ -231,16 +232,20 @@ def _generateItemSvg(item, nodes=None, root=None):
         g1 = doc.getElementsByTagName('g')[0]
         ## get list of sub-groups
         g2 = [n for n in g1.childNodes if isinstance(n, xml.Element) and n.tagName == 'g']
+        
+        defs = doc.getElementsByTagName('defs')
+        if len(defs) > 0:
+            defs = [n for n in defs[0].childNodes if isinstance(n, xml.Element)]
     except:
         print(doc.toxml())
         raise
 
-    prof.mark('render')
+    profiler('render')
 
     ## Get rid of group transformation matrices by applying
     ## transformation to inner coordinates
-    correctCoordinates(g1, item)
-    prof.mark('correct')
+    correctCoordinates(g1, defs, item)
+    profiler('correct')
     ## make sure g1 has the transformation matrix
     #m = (tr.m11(), tr.m12(), tr.m21(), tr.m22(), tr.m31(), tr.m32())
     #g1.setAttribute('transform', "matrix(%f,%f,%f,%f,%f,%f)" % m)
@@ -276,7 +281,9 @@ def _generateItemSvg(item, nodes=None, root=None):
             path = QtGui.QGraphicsPathItem(item.mapToScene(item.shape()))
             item.scene().addItem(path)
             try:
-                pathNode = _generateItemSvg(path, root=root).getElementsByTagName('path')[0]
+                #pathNode = _generateItemSvg(path, root=root).getElementsByTagName('path')[0]
+                pathNode = _generateItemSvg(path, root=root)[0].getElementsByTagName('path')[0]
+                # assume <defs> for this path is empty.. possibly problematic.
             finally:
                 item.scene().removeItem(path)
             
@@ -290,20 +297,24 @@ def _generateItemSvg(item, nodes=None, root=None):
             childGroup = g1.ownerDocument.createElement('g')
             childGroup.setAttribute('clip-path', 'url(#%s)' % clip)
             g1.appendChild(childGroup)
-    prof.mark('clipping')
+    profiler('clipping')
             
     ## Add all child items as sub-elements.
     childs.sort(key=lambda c: c.zValue())
     for ch in childs:
-        cg = _generateItemSvg(ch, nodes, root)
-        if cg is None:
+        csvg = _generateItemSvg(ch, nodes, root)
+        if csvg is None:
             continue
+        cg, cdefs = csvg
         childGroup.appendChild(cg)  ### this isn't quite right--some items draw below their parent (good enough for now)
-    prof.mark('children')
-    prof.finish()
-    return g1
+        defs.extend(cdefs)
+        
+    profiler('children')
+    return g1, defs
 
-def correctCoordinates(node, item):
+def correctCoordinates(node, defs, item):
+    # TODO: correct gradient coordinates inside defs
+    
     ## Remove transformation matrices from <g> tags by applying matrix to coordinates inside.
     ## Each item is represented by a single top-level group with one or more groups inside.
     ## Each inner group contains one or more drawing primitives, possibly of different types.
@@ -351,7 +362,7 @@ def correctCoordinates(node, item):
             if ch.tagName == 'polyline':
                 removeTransform = True
                 coords = np.array([[float(a) for a in c.split(',')] for c in ch.getAttribute('points').strip().split(' ')])
-                coords = pg.transformCoordinates(tr, coords, transpose=True)
+                coords = fn.transformCoordinates(tr, coords, transpose=True)
                 ch.setAttribute('points', ' '.join([','.join([str(a) for a in c]) for c in coords]))
             elif ch.tagName == 'path':
                 removeTransform = True
@@ -366,7 +377,7 @@ def correctCoordinates(node, item):
                         x = x[1:]
                     else:
                         t = ''
-                    nc = pg.transformCoordinates(tr, np.array([[float(x),float(y)]]), transpose=True)
+                    nc = fn.transformCoordinates(tr, np.array([[float(x),float(y)]]), transpose=True)
                     newCoords += t+str(nc[0,0])+','+str(nc[0,1])+' '
                 ch.setAttribute('d', newCoords)
             elif ch.tagName == 'text':
@@ -376,7 +387,7 @@ def correctCoordinates(node, item):
                     #[float(ch.getAttribute('x')), float(ch.getAttribute('y'))], 
                     #[float(ch.getAttribute('font-size')), 0], 
                     #[0,0]])
-                #c = pg.transformCoordinates(tr, c, transpose=True)
+                #c = fn.transformCoordinates(tr, c, transpose=True)
                 #ch.setAttribute('x', str(c[0,0]))
                 #ch.setAttribute('y', str(c[0,1]))
                 #fs = c[1]-c[2]
@@ -398,13 +409,17 @@ def correctCoordinates(node, item):
             ## correct line widths if needed
             if removeTransform and ch.getAttribute('vector-effect') != 'non-scaling-stroke':
                 w = float(grp.getAttribute('stroke-width'))
-                s = pg.transformCoordinates(tr, np.array([[w,0], [0,0]]), transpose=True)
+                s = fn.transformCoordinates(tr, np.array([[w,0], [0,0]]), transpose=True)
                 w = ((s[0]-s[1])**2).sum()**0.5
                 ch.setAttribute('stroke-width', str(w))
             
         if removeTransform:
             grp.removeAttribute('transform')
 
+
+SVGExporter.register()        
+
+
 def itemTransform(item, root):
     ## Return the transformation mapping item to root
     ## (actually to parent coordinate system of root)
@@ -440,35 +455,9 @@ def itemTransform(item, root):
             tr = item.sceneTransform()
         else:
             tr = itemTransform(nextRoot, root) * item.itemTransform(nextRoot)[0]
-            #pos = QtGui.QTransform()
-            #pos.translate(root.pos().x(), root.pos().y())
-            #tr = pos * root.transform() * item.itemTransform(root)[0]
-        
     
     return tr
 
-
-#def correctStroke(node, item, root, width=1):
-    ##print "==============", item, node
-    #if node.hasAttribute('stroke-width'):
-        #width = float(node.getAttribute('stroke-width'))
-    #if node.getAttribute('vector-effect') == 'non-scaling-stroke':
-        #node.removeAttribute('vector-effect')
-        #if isinstance(root, QtGui.QGraphicsScene):
-            #w = item.mapFromScene(pg.Point(width,0))
-            #o = item.mapFromScene(pg.Point(0,0))
-        #else:
-            #w = item.mapFromItem(root, pg.Point(width,0))
-            #o = item.mapFromItem(root, pg.Point(0,0))
-        #w = w-o
-        ##print "   ", w, o, w-o
-        #w = (w.x()**2 + w.y()**2) ** 0.5
-        ##print "   ", w
-        #node.setAttribute('stroke-width', str(w))
-    
-    #for ch in node.childNodes:
-        #if isinstance(ch, xml.Element):
-            #correctStroke(ch, item, root, width)
             
 def cleanXml(node):
     ## remove extraneous text; let the xml library do the formatting.
