@@ -12,32 +12,28 @@ Widget used for displaying 2D or 3D data. Features:
   - ROI plotting
   - Image normalization through a variety of methods
 """
-from pyqtgraph.Qt import QtCore, QtGui, USE_PYSIDE
+import os, sys
+import numpy as np
 
+from ..Qt import QtCore, QtGui, USE_PYSIDE
 if USE_PYSIDE:
     from .ImageViewTemplate_pyside import *
 else:
     from .ImageViewTemplate_pyqt import *
     
-from pyqtgraph.graphicsItems.ImageItem import *
-from pyqtgraph.graphicsItems.ROI import *
-from pyqtgraph.graphicsItems.LinearRegionItem import *
-from pyqtgraph.graphicsItems.InfiniteLine import *
-from pyqtgraph.graphicsItems.ViewBox import *
-#from widgets import ROI
-import sys
-#from numpy import ndarray
-import pyqtgraph.ptime as ptime
-import numpy as np
-import pyqtgraph.debug as debug
+from ..graphicsItems.ImageItem import *
+from ..graphicsItems.ROI import *
+from ..graphicsItems.LinearRegionItem import *
+from ..graphicsItems.InfiniteLine import *
+from ..graphicsItems.ViewBox import *
+from .. import ptime as ptime
+from .. import debug as debug
+from ..SignalProxy import SignalProxy
 
-from pyqtgraph.SignalProxy import SignalProxy
-
-#try:
-    #import pyqtgraph.metaarray as metaarray
-    #HAVE_METAARRAY = True
-#except:
-    #HAVE_METAARRAY = False
+try:
+    from bottleneck import nanmin, nanmax
+except ImportError:
+    from numpy import nanmin, nanmax
     
 
 class PlotROI(ROI):
@@ -67,6 +63,16 @@ class ImageView(QtGui.QWidget):
         imv = pg.ImageView()
         imv.show()
         imv.setImage(data)
+        
+    **Keyboard interaction**
+    
+    * left/right arrows step forward/backward 1 frame when pressed,
+      seek at 20fps when held.
+    * up/down arrows seek at 100fps
+    * pgup/pgdn seek at 1000fps
+    * home/end seek immediately to the first/last frame
+    * space begins playing frames. If time values (in seconds) are given 
+      for each frame, then playback is in realtime.
     """
     sigTimeChanged = QtCore.Signal(object, object)
     sigProcessingChanged = QtCore.Signal(object)
@@ -74,8 +80,31 @@ class ImageView(QtGui.QWidget):
     def __init__(self, parent=None, name="ImageView", view=None, imageItem=None, *args):
         """
         By default, this class creates an :class:`ImageItem <pyqtgraph.ImageItem>` to display image data
-        and a :class:`ViewBox <pyqtgraph.ViewBox>` to contain the ImageItem. Custom items may be given instead 
-        by specifying the *view* and/or *imageItem* arguments.
+        and a :class:`ViewBox <pyqtgraph.ViewBox>` to contain the ImageItem. 
+        
+        ============= =========================================================
+        **Arguments** 
+        parent        (QWidget) Specifies the parent widget to which
+                      this ImageView will belong. If None, then the ImageView
+                      is created with no parent.
+        name          (str) The name used to register both the internal ViewBox
+                      and the PlotItem used to display ROI data. See the *name*
+                      argument to :func:`ViewBox.__init__() 
+                      <pyqtgraph.ViewBox.__init__>`.
+        view          (ViewBox or PlotItem) If specified, this will be used
+                      as the display area that contains the displayed image. 
+                      Any :class:`ViewBox <pyqtgraph.ViewBox>`, 
+                      :class:`PlotItem <pyqtgraph.PlotItem>`, or other 
+                      compatible object is acceptable.
+        imageItem     (ImageItem) If specified, this object will be used to
+                      display the image. Must be an instance of ImageItem
+                      or other compatible object.
+        ============= =========================================================
+        
+        Note: to display axis ticks inside the ImageView, instantiate it 
+        with a PlotItem instance as its view::
+                
+            pg.ImageView(view=pg.PlotItem())
         """
         QtGui.QWidget.__init__(self, parent, *args)
         self.levelMax = 4096
@@ -107,6 +136,8 @@ class ImageView(QtGui.QWidget):
         
         self.ui.histogram.setImageItem(self.imageItem)
         
+        self.menu = None
+        
         self.ui.normGroup.hide()
 
         self.roi = PlotROI(10)
@@ -147,7 +178,8 @@ class ImageView(QtGui.QWidget):
         self.timeLine.sigPositionChanged.connect(self.timeLineChanged)
         self.ui.roiBtn.clicked.connect(self.roiClicked)
         self.roi.sigRegionChanged.connect(self.roiChanged)
-        self.ui.normBtn.toggled.connect(self.normToggled)
+        #self.ui.normBtn.toggled.connect(self.normToggled)
+        self.ui.menuBtn.clicked.connect(self.menuClicked)
         self.ui.normDivideRadio.clicked.connect(self.normRadioChanged)
         self.ui.normSubtractRadio.clicked.connect(self.normRadioChanged)
         self.ui.normOffRadio.clicked.connect(self.normRadioChanged)
@@ -160,6 +192,7 @@ class ImageView(QtGui.QWidget):
         self.normRoi.sigRegionChangeFinished.connect(self.updateNorm)
         
         self.ui.roiPlot.registerPlot(self.name + '_ROI')
+        self.view.register(self.name)
         
         self.noRepeatKeys = [QtCore.Qt.Key_Right, QtCore.Qt.Key_Left, QtCore.Qt.Key_Up, QtCore.Qt.Key_Down, QtCore.Qt.Key_PageUp, QtCore.Qt.Key_PageDown]
         
@@ -190,14 +223,20 @@ class ImageView(QtGui.QWidget):
                            image data.
         ================== =======================================================================
         """
-        prof = debug.Profiler('ImageView.setImage', disabled=True)
+        profiler = debug.Profiler()
         
         if hasattr(img, 'implements') and img.implements('MetaArray'):
             img = img.asarray()
         
         if not isinstance(img, np.ndarray):
-            raise Exception("Image must be specified as ndarray.")
+            required = ['dtype', 'max', 'min', 'ndim', 'shape', 'size']
+            if not all([hasattr(img, attr) for attr in required]):
+                raise TypeError("Image must be NumPy array or any object "
+                                "that provides compatible attributes/methods:\n"
+                                "  %s" % str(required))
+        
         self.image = img
+        self.imageDisp = None
         
         if xvals is not None:
             self.tVals = xvals
@@ -209,7 +248,7 @@ class ImageView(QtGui.QWidget):
         else:
             self.tVals = np.arange(img.shape[0])
         
-        prof.mark('1')
+        profiler()
         
         if axes is None:
             if img.ndim == 2:
@@ -234,13 +273,9 @@ class ImageView(QtGui.QWidget):
             
         for x in ['t', 'x', 'y', 'c']:
             self.axes[x] = self.axes.get(x, None)
-        prof.mark('2')
-            
-        self.imageDisp = None
-        
-        
-        prof.mark('3')
-        
+
+        profiler()
+
         self.currentIndex = 0
         self.updateImage(autoHistogramRange=autoHistogramRange)
         if levels is None and autoLevels:
@@ -250,9 +285,9 @@ class ImageView(QtGui.QWidget):
             
         if self.ui.roiBtn.isChecked():
             self.roiChanged()
-        prof.mark('4')
-            
-            
+
+        profiler()
+
         if self.axes['t'] is not None:
             #self.ui.roiPlot.show()
             self.ui.roiPlot.setXRange(self.tVals.min(), self.tVals.max())
@@ -271,8 +306,8 @@ class ImageView(QtGui.QWidget):
                 s.setBounds([start, stop])
         #else:
             #self.ui.roiPlot.hide()
-        prof.mark('5')
-            
+        profiler()
+
         self.imageItem.resetTransform()
         if scale is not None:
             self.imageItem.scale(*scale)
@@ -280,14 +315,18 @@ class ImageView(QtGui.QWidget):
             self.imageItem.setPos(*pos)
         if transform is not None:
             self.imageItem.setTransform(transform)
-        prof.mark('6')
-            
+
+        profiler()
+
         if autoRange:
             self.autoRange()
         self.roiClicked()
-        prof.mark('7')
-        prof.finish()
 
+        profiler()
+
+    def clear(self):
+        self.image = None
+        self.imageItem.clear()
         
     def play(self, rate):
         """Begin automatically stepping frames forward at the given rate (in fps).
@@ -311,7 +350,7 @@ class ImageView(QtGui.QWidget):
         self.ui.histogram.setLevels(min, max)
 
     def autoRange(self):
-        """Auto scale and pan the view around the image."""
+        """Auto scale and pan the view around the image such that the image fills the view."""
         image = self.getProcessedImage()
         self.view.autoRange()
         
@@ -322,11 +361,10 @@ class ImageView(QtGui.QWidget):
         if self.imageDisp is None:
             image = self.normalize(self.image)
             self.imageDisp = image
-            self.levelMin, self.levelMax = list(map(float, ImageView.quickMinMax(self.imageDisp)))
+            self.levelMin, self.levelMax = list(map(float, self.quickMinMax(self.imageDisp)))
             
         return self.imageDisp
         
-        
     def close(self):
         """Closes the widget nicely, making sure to clear the graphics scene and release memory."""
         self.ui.roiPlot.close()
@@ -378,7 +416,6 @@ class ImageView(QtGui.QWidget):
         else:
             QtGui.QWidget.keyReleaseEvent(self, ev)
         
-        
     def evalKeyState(self):
         if len(self.keysPressed) == 1:
             key = list(self.keysPressed.keys())[0]
@@ -402,16 +439,13 @@ class ImageView(QtGui.QWidget):
         else:
             self.play(0)
         
-        
     def timeout(self):
         now = ptime.time()
         dt = now - self.lastPlayTime
         if dt < 0:
             return
         n = int(self.playRate * dt)
-        #print n, dt
         if n != 0:
-            #print n, dt, self.lastPlayTime
             self.lastPlayTime += (float(n)/self.playRate)
             if self.currentIndex+n > self.image.shape[0]:
                 self.play(0)
@@ -436,17 +470,14 @@ class ImageView(QtGui.QWidget):
         self.autoLevels()
         self.roiChanged()
         self.sigProcessingChanged.emit(self)
-        
     
     def updateNorm(self):
         if self.ui.normTimeRangeCheck.isChecked():
-            #print "show!"
             self.normRgn.show()
         else:
             self.normRgn.hide()
         
         if self.ui.normROICheck.isChecked():
-            #print "show!"
             self.normRoi.show()
         else:
             self.normRoi.hide()
@@ -522,21 +553,25 @@ class ImageView(QtGui.QWidget):
                 coords = coords - coords[:,0,np.newaxis]
                 xvals = (coords**2).sum(axis=0) ** 0.5
                 self.roiCurve.setData(y=data, x=xvals)
-                
-            #self.ui.roiPlot.replot()
 
-
-    @staticmethod
-    def quickMinMax(data):
+    def quickMinMax(self, data):
+        """
+        Estimate the min/max values of *data* by subsampling.
+        """
         while data.size > 1e6:
             ax = np.argmax(data.shape)
             sl = [slice(None)] * data.ndim
             sl[ax] = slice(None, None, 2)
             data = data[sl]
-        return data.min(), data.max()
+        return nanmin(data), nanmax(data)
 
     def normalize(self, image):
+        """
+        Process *image* using the normalization options configured in the
+        control panel.
         
+        This can be repurposed to process any data through the same filter.
+        """
         if self.ui.normOffRadio.isChecked():
             return image
             
@@ -643,3 +678,43 @@ class ImageView(QtGui.QWidget):
     def getHistogramWidget(self):
         """Return the HistogramLUTWidget for this ImageView"""
         return self.ui.histogram
+
+    def export(self, fileName):
+        """
+        Export data from the ImageView to a file, or to a stack of files if
+        the data is 3D. Saving an image stack will result in index numbers
+        being added to the file name. Images are saved as they would appear
+        onscreen, with levels and lookup table applied.
+        """
+        img = self.getProcessedImage()
+        if self.hasTimeAxis():
+            base, ext = os.path.splitext(fileName)
+            fmt = "%%s%%0%dd%%s" % int(np.log10(img.shape[0])+1)
+            for i in range(img.shape[0]):
+                self.imageItem.setImage(img[i], autoLevels=False)
+                self.imageItem.save(fmt % (base, i, ext))
+            self.updateImage()
+        else:
+            self.imageItem.save(fileName)
+            
+    def exportClicked(self):
+        fileName = QtGui.QFileDialog.getSaveFileName()
+        if fileName == '':
+            return
+        self.export(fileName)
+        
+    def buildMenu(self):
+        self.menu = QtGui.QMenu()
+        self.normAction = QtGui.QAction("Normalization", self.menu)
+        self.normAction.setCheckable(True)
+        self.normAction.toggled.connect(self.normToggled)
+        self.menu.addAction(self.normAction)
+        self.exportAction = QtGui.QAction("Export", self.menu)
+        self.exportAction.triggered.connect(self.exportClicked)
+        self.menu.addAction(self.exportAction)
+        
+    def menuClicked(self):
+        if self.menu is None:
+            self.buildMenu()
+        self.menu.popup(QtGui.QCursor.pos())
+        
