@@ -1,5 +1,5 @@
-from pyqtgraph.Qt import QtGui
-import pyqtgraph.functions as fn
+from ..Qt import QtGui
+from .. import functions as fn
 import numpy as np
 
 class MeshData(object):
@@ -23,18 +23,18 @@ class MeshData(object):
 
     def __init__(self, vertexes=None, faces=None, edges=None, vertexColors=None, faceColors=None):
         """
-        ============= =====================================================
-        Arguments
-        vertexes      (Nv, 3) array of vertex coordinates. 
-                      If faces is not specified, then this will instead be
-                      interpreted as (Nf, 3, 3) array of coordinates.
-        faces         (Nf, 3) array of indexes into the vertex array.
-        edges         [not available yet]
-        vertexColors  (Nv, 4) array of vertex colors. 
-                      If faces is not specified, then this will instead be
-                      interpreted as (Nf, 3, 4) array of colors.
-        faceColors    (Nf, 4) array of face colors.
-        ============= =====================================================
+        ==============  =====================================================
+        **Arguments:**
+        vertexes        (Nv, 3) array of vertex coordinates.
+                        If faces is not specified, then this will instead be
+                        interpreted as (Nf, 3, 3) array of coordinates.
+        faces           (Nf, 3) array of indexes into the vertex array.
+        edges           [not available yet]
+        vertexColors    (Nv, 4) array of vertex colors.
+                        If faces is not specified, then this will instead be
+                        interpreted as (Nf, 3, 4) array of colors.
+        faceColors      (Nf, 4) array of face colors.
+        ==============  =====================================================
         
         All arguments are optional.
         """
@@ -84,64 +84,11 @@ class MeshData(object):
                 if faceColors is not None:
                     self.setFaceColors(faceColors)
             
-            #self.setFaces(vertexes=vertexes, faces=faces, vertexColors=vertexColors, faceColors=faceColors)
-            
-        
-    #def setFaces(self, vertexes=None, faces=None, vertexColors=None, faceColors=None):
-        #"""
-        #Set the faces in this data set.
-        #Data may be provided either as an Nx3x3 array of floats (9 float coordinate values per face)::
-        
-            #faces = [ [(x, y, z), (x, y, z), (x, y, z)], ... ] 
-            
-        #or as an Nx3 array of ints (vertex integers) AND an Mx3 array of floats (3 float coordinate values per vertex)::
-        
-            #faces = [ (p1, p2, p3), ... ]
-            #vertexes = [ (x, y, z), ... ]
-            
-        #"""
-        #if not isinstance(vertexes, np.ndarray):
-            #vertexes = np.array(vertexes)
-        #if vertexes.dtype != np.float:
-            #vertexes = vertexes.astype(float)
-        #if faces is None:
-            #self._setIndexedFaces(vertexes, vertexColors, faceColors)
-        #else:
-            #self._setUnindexedFaces(faces, vertexes, vertexColors, faceColors)
-        ##print self.vertexes().shape
-        ##print self.faces().shape
-        
-    
-    #def setMeshColor(self, color):
-        #"""Set the color of the entire mesh. This removes any per-face or per-vertex colors."""
-        #color = fn.Color(color)
-        #self._meshColor = color.glColor()
-        #self._vertexColors = None
-        #self._faceColors = None
-    
-        
-    #def __iter__(self):
-        #"""Iterate over all faces, yielding a list of three tuples [(position, normal, color), ...] for each face."""
-        #vnorms = self.vertexNormals()
-        #vcolors = self.vertexColors()
-        #for i in range(self._faces.shape[0]):
-            #face = []
-            #for j in [0,1,2]:
-                #vind = self._faces[i,j]
-                #pos = self._vertexes[vind]
-                #norm = vnorms[vind]
-                #if vcolors is None:
-                    #color = self._meshColor
-                #else:
-                    #color = vcolors[vind]
-                #face.append((pos, norm, color))
-            #yield face
-    
-    #def __len__(self):
-        #return len(self._faces)
-    
     def faces(self):
-        """Return an array (Nf, 3) of vertex indexes, three per triangular face in the mesh."""
+        """Return an array (Nf, 3) of vertex indexes, three per triangular face in the mesh.
+        
+        If faces have not been computed for this mesh, the function returns None.
+        """
         return self._faces
     
     def edges(self):
@@ -161,8 +108,6 @@ class MeshData(object):
         self.resetNormals()
         self._vertexColorsIndexedByFaces = None
         self._faceColorsIndexedByFaces = None
-        
-        
     
     def vertexes(self, indexed=None):
         """Return an array (N,3) of the positions of vertexes in the mesh. 
@@ -207,7 +152,6 @@ class MeshData(object):
         self._vertexNormalsIndexedByFaces = None
         self._faceNormals = None
         self._faceNormalsIndexedByFaces = None
-            
         
     def hasFaceIndexedData(self):
         """Return True if this object already has vertex positions indexed by face"""
@@ -229,7 +173,6 @@ class MeshData(object):
             if v is not None:
                 return True
         return False
-        
     
     def faceNormals(self, indexed=None):
         """
@@ -242,7 +185,6 @@ class MeshData(object):
             v = self.vertexes(indexed='faces')
             self._faceNormals = np.cross(v[:,1]-v[:,0], v[:,2]-v[:,0])
         
-        
         if indexed is None:
             return self._faceNormals
         elif indexed == 'faces':
@@ -266,7 +208,11 @@ class MeshData(object):
             vertFaces = self.vertexFaces()
             self._vertexNormals = np.empty(self._vertexes.shape, dtype=float)
             for vindex in xrange(self._vertexes.shape[0]):
-                norms = faceNorms[vertFaces[vindex]]  ## get all face normals
+                faces = vertFaces[vindex]
+                if len(faces) == 0:
+                    self._vertexNormals[vindex] = (0,0,0)
+                    continue
+                norms = faceNorms[faces]  ## get all face normals
                 norm = norms.sum(axis=0)       ## sum normals
                 norm /= (norm**2).sum()**0.5  ## and re-normalize
                 self._vertexNormals[vindex] = norm
@@ -363,7 +309,6 @@ class MeshData(object):
         ## This is done by collapsing into a list of 'unique' vertexes (difference < 1e-14) 
         
         ## I think generally this should be discouraged..
-        
         faces = self._vertexesIndexedByFaces
         verts = {}  ## used to remember the index of each vertex position
         self._faces = np.empty(faces.shape[:2], dtype=np.uint)
@@ -403,12 +348,10 @@ class MeshData(object):
         Return list mapping each vertex index to a list of face indexes that use the vertex.
         """
         if self._vertexFaces is None:
-            self._vertexFaces = [None] * len(self.vertexes())
+            self._vertexFaces = [[] for i in xrange(len(self.vertexes()))]
             for i in xrange(self._faces.shape[0]):
                 face = self._faces[i]
                 for ind in face:
-                    if self._vertexFaces[ind] is None:
-                        self._vertexFaces[ind] = []  ## need a unique/empty list to fill
                     self._vertexFaces[ind].append(i)
         return self._vertexFaces
         
@@ -426,22 +369,35 @@ class MeshData(object):
         #pass
         
     def _computeEdges(self):
-        ## generate self._edges from self._faces
-        #print self._faces
-        nf = len(self._faces)
-        edges = np.empty(nf*3, dtype=[('i', np.uint, 2)])
-        edges['i'][0:nf] = self._faces[:,:2]
-        edges['i'][nf:2*nf] = self._faces[:,1:3]
-        edges['i'][-nf:,0] = self._faces[:,2]
-        edges['i'][-nf:,1] = self._faces[:,0]
-        
-        # sort per-edge
-        mask = edges['i'][:,0] > edges['i'][:,1]
-        edges['i'][mask] = edges['i'][mask][:,::-1]
-        
-        # remove duplicate entries
-        self._edges = np.unique(edges)['i']
-        #print self._edges
+        if not self.hasFaceIndexedData:
+            ## generate self._edges from self._faces
+            nf = len(self._faces)
+            edges = np.empty(nf*3, dtype=[('i', np.uint, 2)])
+            edges['i'][0:nf] = self._faces[:,:2]
+            edges['i'][nf:2*nf] = self._faces[:,1:3]
+            edges['i'][-nf:,0] = self._faces[:,2]
+            edges['i'][-nf:,1] = self._faces[:,0]
+            
+            # sort per-edge
+            mask = edges['i'][:,0] > edges['i'][:,1]
+            edges['i'][mask] = edges['i'][mask][:,::-1]
+            
+            # remove duplicate entries
+            self._edges = np.unique(edges)['i']
+            #print self._edges
+        elif self._vertexesIndexedByFaces is not None:
+            verts = self._vertexesIndexedByFaces
+            edges = np.empty((verts.shape[0], 3, 2), dtype=np.uint)
+            nf = verts.shape[0]
+            edges[:,0,0] = np.arange(nf) * 3
+            edges[:,0,1] = edges[:,0,0] + 1
+            edges[:,1,0] = edges[:,0,1]
+            edges[:,1,1] = edges[:,1,0] + 1
+            edges[:,2,0] = edges[:,1,1]
+            edges[:,2,1] = edges[:,0,0]
+            self._edges = edges
+        else:
+            raise Exception("MeshData cannot generate edges--no faces in this data.")
         
         
     def save(self):
@@ -516,4 +472,33 @@ class MeshData(object):
         
         return MeshData(vertexes=verts, faces=faces)
         
-        
\ No newline at end of file
+    @staticmethod
+    def cylinder(rows, cols, radius=[1.0, 1.0], length=1.0, offset=False):
+        """
+        Return a MeshData instance with vertexes and faces computed
+        for a cylindrical surface.
+        The cylinder may be tapered with different radii at each end (truncated cone)
+        """
+        verts = np.empty((rows+1, cols, 3), dtype=float)
+        if isinstance(radius, int):
+            radius = [radius, radius] # convert to list
+        ## compute vertexes
+        th = np.linspace(2 * np.pi, 0, cols).reshape(1, cols)
+        r = np.linspace(radius[0],radius[1],num=rows+1, endpoint=True).reshape(rows+1, 1) # radius as a function of z
+        verts[...,2] = np.linspace(0, length, num=rows+1, endpoint=True).reshape(rows+1, 1) # z
+        if offset:
+            th = th + ((np.pi / cols) * np.arange(rows+1).reshape(rows+1,1))  ## rotate each row by 1/2 column
+        verts[...,0] = r * np.cos(th) # x = r cos(th)
+        verts[...,1] = r * np.sin(th) # y = r sin(th)
+        verts = verts.reshape((rows+1)*cols, 3) # just reshape: no redundant vertices...
+        ## compute faces
+        faces = np.empty((rows*cols*2, 3), dtype=np.uint)
+        rowtemplate1 = ((np.arange(cols).reshape(cols, 1) + np.array([[0, 1, 0]])) % cols) + np.array([[0, 0, cols]])
+        rowtemplate2 = ((np.arange(cols).reshape(cols, 1) + np.array([[0, 1, 1]])) % cols) + np.array([[cols, 0, cols]])
+        for row in range(rows):
+            start = row * cols * 2 
+            faces[start:start+cols] = rowtemplate1 + row * cols
+            faces[start+cols:start+(cols*2)] = rowtemplate2 + row * cols
+        
+        return MeshData(vertexes=verts, faces=faces)
+        
